#include "location.h"

#include "location_p.h"

#include "ytv.h"

#include <QString>
#include <QObject>
#include <QNetworkAccessManager>
#include <QUrl>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QXmlStreamReader>
#include <QDebug>
#include <QXmlStreamAttributes>
#include <QStringRef>
#include <QGeoPositionInfo>

QTM_USE_NAMESPACE

Location::Location(const QString &x, const QString &y, const QString &label) :
    q(new LocationPrivate(x, y, label)),
    manager(new QNetworkAccessManager(this))
{
    connect(
                manager, SIGNAL(finished(QNetworkReply*)),
                this, SLOT(replyFinished(QNetworkReply*))
                );
}

Location::Location(const QGeoPositionInfo &positionInfo, const QString &label) :
    q(new LocationPrivate(label)),
    manager(0)
{
    setLocation(positionInfo);
}

void Location::setLocation(const QGeoPositionInfo &positionInfo)
{
    double latitude = positionInfo.coordinate().latitude();
    double longitude = positionInfo.coordinate().longitude();

    qDebug() << "GPS Location/" << longitude << latitude;

    q->setLongitude(longitude);
    q->setlatitude(latitude);
    q->setValid(true);
}

Location::Location(const Location &from) :
    QObject(0),
    q(new LocationPrivate(from.label())),
    manager(0)
{
    q->setAddress(from.address());
    q->setLongitude(from.longitude());
    q->setlatitude(from.latitude());
    q->setValid(from.isValid());
    if (from.manager != 0) {
        manager = new QNetworkAccessManager(this);
        connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    }
}

Location::Location(const QString &label) :
    q(new LocationPrivate(label)),
    manager(new QNetworkAccessManager(this))
{
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
}

Location::~Location()
{
    delete q;
    q=0;
    delete manager;
    manager=0;
}

Location &Location::operator=(const Location &from)
{
    q = new LocationPrivate(from.label());
    q->setAddress(from.address());
    q->setLongitude(from.longitude());
    q->setlatitude(from.latitude());
    q->setValid(from.isValid());

    if (from.manager != 0) {
        manager = new QNetworkAccessManager(this);
        connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    } else {
        manager = 0;
    }

    return *this;
}

void Location::resolveAddress(const QString &address)
{
    qDebug() << "resolving address (" << address << ")";

    q->setAddress(address);
    q->setValid(false);

    QUrl fullUrl(Ytv::Url);
    QByteArray encodedAddress = QUrl::toPercentEncoding(address);
    qDebug() << "encodedAddress/" << encodedAddress;

    fullUrl.addQueryItem("user", Ytv::Username);
    fullUrl.addQueryItem("pass", Ytv::Password);
    fullUrl.addEncodedQueryItem("key", encodedAddress);
    fullUrl.addQueryItem("request", Ytv::Geocode);
    fullUrl.addQueryItem("format", Ytv::Xml);
    fullUrl.addQueryItem("language", Ytv::English);
    fullUrl.addQueryItem("epsg_out", Ytv::WGS84);

    qDebug() << "resolveAddress/" << address << "/" << fullUrl;
    manager->get(QNetworkRequest(fullUrl));
    qDebug() << "waiting for reply from Ytv";
    emit busy(true);
}

void Location::replyFinished(QNetworkReply * reply)
{
    qDebug() << "address resolved";
    q->parseReply(reply->readAll());

    if (isValid()) {
        qDebug() << label() << "becomeValid";
        emit becomeValid();
    } else {
        qDebug() << label() << "not valid";
        emit becomeInValid();
    }

    emit(busy(false));
}

QString Location::longitude() const
{
    return q->longitude();
}

QString Location::latitude() const
{
    return q->latitude();
}

void Location::setLabel(const QString &label) const
{
    q->setLabel(label);
}

void Location::setValid(bool valid) const
{
    q->setValid(valid);
}

QString Location::label() const
{
    return q->label();
}

void Location::setAddress(const QString &address) const
{
    qDebug() << "setting address to" << address;
    q->setAddress(address);
}

QString Location::address() const
{
    return q->address();
}

bool Location::isValid() const
{
    return q->isValid();
}
